@@ -15,8 +15,8 @@ __email__ = 'pablo@ppizarror.com'
 __keywords__ = 'latex detex parser gui'
 __license__ = 'MIT'
 __module_name__ = 'pydetex'
-__url__ = 'https://github.com/ppizarror/PyDetex'
-__url_bug_tracker__ = 'https://github.com/ppizarror/PyDetex'
-__url_documentation__ = 'https://github.com/ppizarror/PyDetex'
+__url__ = 'https://pydetex.readthedocs.io'
+__url_bug_tracker__ = 'https://github.com/ppizarror/PyDetex/issues'
+__url_documentation__ = 'https://pydetex.readthedocs.io'
 __url_source_code__ = 'https://github.com/ppizarror/PyDetex'
 __version__ = pydetex.version.ver
