@@ -39,6 +39,8 @@ def _append_to_datas(file_path: str, target_folder: str, base_target_folder: str
         target_folder = os.path.basename(os.path.dirname(res_path))
     if os.path.exists(res_path):
         datas.append((res_path, os.path.join(base_target_folder, target_folder)))
+    else:
+        raise FileNotFoundError(f'{file_path} does not exist')
 
 
 # Append resources
@@ -47,7 +49,18 @@ res_files = [
     'res/dictionary.ico',
     'res/icon.gif',
     'res/icon.ico',
-    'res/stopwords.json'
+    'res/placeholder_en.tex',
+    'res/placeholder_es.tex',
+    'res/stopwords.json',
+    'res/u_subscripts.txt',
+    'res/u_superscripts.txt',
+    'res/u_symbols.txt',
+    'res/u_textbb.txt',
+    'res/u_textbf.txt',
+    'res/u_textcal.txt',
+    'res/u_textfrak.txt',
+    'res/u_textit.txt',
+    'res/u_textmono.txt'
 ]
 
 for f in res_files:
