@@ -22,6 +22,7 @@ REPLACE_SYMBOLS_LIBRARY: List[Tuple[str, str]] = [
     # Letters
     ('--', '–'),
     ('---', '—'),
+    ('\&', '&'),
     ('~', ' '),
     ('ﬁ', 'fi')
 ]
@@ -121,7 +122,6 @@ REPLACE_EQUATION_SYMBOLS_LIBRARY: List[Tuple[str, str]] = [
 REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\AC', '∿'),
     ('\\aleph', 'ℵ'),
-    ('\\Alpha', 'Α'),
     ('\\alpha', 'α'),
     ('\\amalg', '⨿'),
     ('\\angle', '∠'),
@@ -135,7 +135,6 @@ REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\backsimeq', '⋍'),
     ('\\barwedge', '⊼'),
     ('\\because', '∵'),
-    ('\\Beta', 'Β'),
     ('\\beta', 'β'),
     ('\\beth', 'ℶ'),
     ('\\between', '≬'),
@@ -162,7 +161,6 @@ REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\cdot', '·'),
     ('\\cdots', '⋯'),
     ('\\checkmark', '✓'),
-    ('\\Chi', 'Χ'),
     ('\\chi', 'χ'),
     ('\\circ', '∘'),
     ('\\circeq', '≗'),
@@ -212,13 +210,11 @@ REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\downharpoonleft', '⇃'),
     ('\\downharpoonright', '⇂'),
     ('\\ell', 'ℓ'),
-    ('\\Epsilon', 'Ε'),
     ('\\epsilon', 'ϵ'),
     ('\\eqcirc', '≖'),
     ('\\eqcolon', '∹'),
     ('\\eqqcolon', '≕'),
     ('\\equiv', '≡'),
-    ('\\Eta', 'Η'),
     ('\\eta', 'η'),
     ('\\Euler', 'ℇ'),
     ('\\euro', '€'),
@@ -277,11 +273,9 @@ REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\int', '∫'),
     ('\\intercal', '⊺'),
     ('\\invamp', '⅋'),
-    ('\\Iota', 'Ι'),
     ('\\iota', 'ι'),
     ('\\jmath', 'ȷ'),
     ('\\Join', '⨝'),
-    ('\\Kappa', 'Κ'),
     ('\\kappa', 'κ'),
     ('\\Koppa', 'Ϟ'),
     ('\\koppa', 'ϟ'),
@@ -419,7 +413,6 @@ REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\mid', '∣'),
     ('\\models', '⊧'),
     ('\\mp', '∓'),
-    ('\\Mu', 'Μ'),
     ('\\mu', 'μ'),
     ('\\multimap', '⊸'),
     ('\\nabla', '∇'),
@@ -471,7 +464,6 @@ REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\nsubseteq', '⊈'),
     ('\\nsucc', '⊁'),
     ('\\nsupseteq', '⊉'),
-    ('\\Nu', 'Ν'),
     ('\\nu', 'ν'),
     ('\\nwarrow', '↖'),
     ('\\odot', '⊙'),
@@ -482,8 +474,6 @@ REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\ointclockwise', '∲'),
     ('\\Omega', 'Ω'),
     ('\\omega', 'ω'),
-    ('\\Omicron', 'Ο'),
-    ('\\omicron', 'ο'),
     ('\\ominus', '⊖'),
     ('\\oplus', '⊕'),
     ('\\oslash', '⊘'),
@@ -518,7 +508,6 @@ REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\Re', 'ℜ'),
     ('\\rfloor', '⌋'),
     ('\\RHD', '‣'),
-    ('\\Rho', 'Ρ'),
     ('\\rho', 'ρ'),
     ('\\rightarrow', '→'),
     ('\\Rightarrow', '⇒'),
@@ -562,6 +551,7 @@ REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\sqsubseteq', '⊑'),
     ('\\sqsubsetneq', '⋤'),
     ('\\sqsupset', '⊐'),
+    ('\\sqsupseteq', '⊒'),
     ('\\sqsupsetneq', '⋥'),
     ('\\square', '□'),
     ('\\stackrel{=}{=}', '≣'),
@@ -588,9 +578,8 @@ REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\supseteq', '⊇'),
     ('\\supsetneq', '⊋'),
     ('\\swarrow', '↙'),
-    ('\\Tau', 'Τ'),
     ('\\tau', 'τ'),
-    ('\\textasciimacron­', '¯'),
+    ('\\textasciimacron', '¯'),
     ('\\textbardbl', '‖'),
     ('\\textbrokenbar', '¦'),
     ('\\textcent', '¢'),
@@ -626,7 +615,6 @@ REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\third', '‴'),
     ('\\times', '×'),
     ('\\top', '⊤'),
-    ('\\to}', '⊒'),
     ('\\triangle', '△'),
     ('\\triangleleft', '⊲'),
     ('\\triangleq', '≜'),
@@ -663,6 +651,5 @@ REPLACE_TEX_COMMANDS_LIBRARY: List[Tuple[str, str]] = [
     ('\\Xi', 'Ξ'),
     ('\\xi', 'ξ'),
     ('\\Yup', '⅄'),
-    ('\\zeta', 'ζ'),
-    ('\\Zeta', 'Ζ')
+    ('\\zeta', 'ζ')
 ]
