@@ -19,6 +19,7 @@ import pyperclip
 import requests
 import string
 import sys
+import traceback
 
 from nltk.tokenize import RegexpTokenizer
 from outdated import check_outdated
@@ -47,13 +48,17 @@ class PyDetexGUI(object):
     """
 
     _cfg: 'Settings'
-    _copy_clip: 'tk.Button'
+    _clear_button: 'ut.Button'
+    _clip: bool
+    _copy_clip_button: 'tk.Button'
     _detect_language_event_id: str
     _detected_lang_tag: str
     _dictionary: 'MultiDictionary'
-    _dictionary_window: Optional['gui_ut.DictionaryGUI']
     _dictionary_btn: 'tk.Button'
+    _dictionary_window: Optional['gui_ut.DictionaryGUI']
     _paste_timeout_error: int
+    _process_button: 'ut.Button'
+    _process_clip_button: 'tk.Button'
     _ready: bool
     _root: 'tk.Tk'
     _settings_window: Optional['gui_ut.SettingsWindow']
@@ -71,7 +76,6 @@ class PyDetexGUI(object):
         """
         Constructor.
         """
-
         # ----------------------------------------------------------------------
         # Creates the window
         # ----------------------------------------------------------------------
@@ -91,7 +95,10 @@ class PyDetexGUI(object):
         # noinspection PyProtectedMember
         self._root.tk.call('wm', 'iconphoto', self._root._w, img)
         if not ut.IS_OSX:
-            self._root.iconbitmap(ut.RESOURCES_PATH + 'icon.ico')
+            try:
+                self._root.iconbitmap(ut.RESOURCES_PATH + 'icon.ico')
+            except tk.TclError:  # Linux
+                pass
 
         self._root.minsize(width=window_size[0], height=window_size[1])
         self._root.resizable(width=False, height=False)
@@ -114,34 +121,41 @@ class PyDetexGUI(object):
                                          command=self._open_dictionary, relief=tk.GROOVE)
         self._dictionary_btn.pack(side=tk.RIGHT, padx=(0, 7 if ut.IS_OSX else 11))
 
-        hthick, hcolor = 3 if ut.IS_OSX else 1, '#426392' if ut.IS_OSX else '#475aff'
-        fsize = self._cfg.get(self._cfg.CFG_FONT_SIZE)
-
         # ----------------------------------------------------------------------
         # Input texts
         # ----------------------------------------------------------------------
+        hthick, hcolor = 3 if ut.IS_OSX else 1, '#426392' if ut.IS_OSX else '#475aff'
+        fsize = self._cfg.get(self._cfg.CFG_FONT_SIZE)
+        show_lnum = self._cfg.get(self._cfg.CFG_SHOW_LINE_NUMBERS)
+        self._tab_spaces = 4
+
         # In text
         f1 = tk.Frame(self._root, border=0, width=window_size[0], height=window_size[2])
         f1.pack(fill='both', padx=10)
         f1.pack_propagate(0)
 
-        self._text_in = gui_ut.RichText(self._cfg, f1, wrap='word', highlightthickness=hthick,
-                                        highlightcolor=hcolor, font_size=fsize, editable=True)
+        self._text_in = gui_ut.RichText(self._cfg, self._root, f1, wrap='word', highlightthickness=hthick,
+                                        highlightcolor=hcolor, font_size=fsize, editable=True,
+                                        scrollbar_y=f1, add_line_numbers=f1 if show_lnum else None)
         self._text_in.pack(fill='both')
         self._text_in.bind('<Button>', self._process_cursor_in)
         self._text_in.bind('<ButtonRelease>', self._process_cursor_in)
         self._text_in.bind('<FocusIn>', self._process_focusin_in)
         self._text_in.bind('<FocusOut>', self._process_focusout_in)
-        self._text_in.bind('<Key>', self._process_in_key)
+        self._text_in.bind('<Key>', self._process_in_key, add='+')
+        self._text_in.bind('<Key-Tab>', self._text_in.tab_selected)
+        self._text_in.bind('<Shift-KeyPress-Tab>', self._text_in.undo_tab_selected)
+        self._text_in.tab_spaces = self._tab_spaces
 
         # Out text
         f2 = tk.Frame(self._root, border=0, width=window_size[0], height=window_size[2])
-        f2.pack(fill='both', padx=10, pady=5)
+        f2.pack(fill='both', padx=10, pady=(window_size[3], 0))
         f2.pack_propagate(0)
 
-        self._text_out = gui_ut.RichText(self._cfg, f2, wrap='word', highlightthickness=hthick,
-                                         highlightcolor=hcolor, font_size=fsize, copy=True)
-        self._text_out.bind('<Key>', self._process_out_key)
+        self._text_out = gui_ut.RichText(self._cfg, self._root, f2, wrap='word', highlightthickness=hthick,
+                                         highlightcolor=hcolor, font_size=fsize, copy=True,
+                                         scrollbar_y=f2, add_line_numbers=f2 if show_lnum else None)
+        self._text_out.bind('<Key>', self._process_out_key, add='+')
         self._text_out.pack(fill='both')
 
         # ----------------------------------------------------------------------
@@ -149,24 +163,28 @@ class PyDetexGUI(object):
         # ----------------------------------------------------------------------
         command_btn_packx = 15  # margin px
         f3 = tk.Frame(self._root, border=2)
-        f3.pack(pady=(9, 18))
+        f3.pack(pady=(window_size[4], window_size[4] + 5))
 
         # Process
-        ut.Button(f3, text=ut.button_text(self._cfg.lang('process')), command=self._process, relief=tk.GROOVE,
-                  bg='#475aff' if ut.IS_OSX else '#6388ff').pack(side=tk.LEFT, padx=(0, command_btn_packx))
+        self._process_button = ut.Button(f3, text=ut.button_text(self._cfg.lang('process')),
+                                         command=self._process, relief=tk.GROOVE,
+                                         bg='#475aff' if ut.IS_OSX else '#6388ff')
+        self._process_button.pack(side=tk.LEFT, padx=(0, command_btn_packx))
 
         # Process clip
-        tk.Button(f3, text=ut.button_text(self._cfg.lang('process_clip')), command=self._process_clip,
-                  relief=tk.GROOVE).pack(side=tk.LEFT, padx=(0, command_btn_packx))
+        self._process_clip_button = tk.Button(f3, text=ut.button_text(self._cfg.lang('process_clip')),
+                                              command=self._process_clip, relief=tk.GROOVE)
+        self._process_clip_button.pack(side=tk.LEFT, padx=(0, command_btn_packx))
 
         # Copy to clip
-        self._copy_clip = tk.Button(f3, text=ut.button_text(self._cfg.lang('process_copy')), command=self._copy_to_clip,
-                                    relief=tk.GROOVE)
-        self._copy_clip.pack(side=tk.LEFT, padx=(0, command_btn_packx))
+        self._copy_clip_button = tk.Button(f3, text=ut.button_text(self._cfg.lang('process_copy')),
+                                           command=self._copy_to_clip, relief=tk.GROOVE)
+        self._copy_clip_button.pack(side=tk.LEFT, padx=(0, command_btn_packx))
 
         # Clear
-        ut.Button(f3, text=ut.button_text(self._cfg.lang('clear')), command=self._clear,
-                  relief=tk.GROOVE, bg='#ff7878').pack(side=tk.LEFT)
+        self._clear_button = ut.Button(f3, text=ut.button_text(self._cfg.lang('clear')), command=self._clear,
+                                       relief=tk.GROOVE, bg='#ff7878')
+        self._clear_button.pack(side=tk.LEFT)
 
         # ----------------------------------------------------------------------
         # Status bar
@@ -208,6 +226,17 @@ class PyDetexGUI(object):
         self._status_clear_event_id = ''
         self._clear()  # This also changes states
 
+        # Check if pyperclip is enabled
+        self._clip = True
+        try:
+            pyperclip.paste()
+        except pyperclip.PyperclipException:
+            self._clip = False
+            error = 'pyperclip is not available on your system (copy/paste mechanism). GUI buttons were disabled'
+            warn(error)
+            self._process_clip_button['state'] = tk.DISABLED
+            self._copy_clip_button['state'] = tk.DISABLED
+
         # Set variables
         self._detect_language_event_id = ''
         self._detected_lang_tag = '–'
@@ -218,6 +247,9 @@ class PyDetexGUI(object):
 
         # Inserts the placeholder text
         self._insert_in(self._cfg.lang('placeholder'))
+
+        # Finals
+        self._text_out['state'] = tk.DISABLED
         self._root.update()
 
     def _open_file(self) -> None:
@@ -233,23 +265,32 @@ class PyDetexGUI(object):
             filetypes=[(self._cfg.lang('open_file_latex_file'), '*.tex')])
         if filename == '':
             return self._status_clear()
-        filename_dir = os.path.dirname(filename)
+        try:
+            filename_dir = os.path.dirname(filename)
+        except TypeError:  # Linux
+            return
         self._cfg.set(self._cfg.CFG_LAST_OPENED_FOLDER, filename_dir)
         try:
             text = ut.open_file(filename)
             self._clear()
             self._insert_in(text)
             self._cfg.save()
+            os.chdir(filename_dir)
         except PermissionError:
             pass
 
-    def _insert_in(self, text: str) -> None:
+    def _insert_in(self, text: str, clear: bool = False) -> None:
         """
         Insert text to in widget.
 
         :param text: Text
+        :param clear: Clear the text
         """
+        if clear:
+            self._text_in.clear()
+        text = text.replace('\t', ' ' * self._tab_spaces)
         self._text_in.insert(tk.END, text)
+        self._text_in.redraw()
         self._detect_language()
         self._process_cursor_in(None)
 
@@ -261,6 +302,7 @@ class PyDetexGUI(object):
         :param clear: Clear after time
         :param clear_time: Clear time in miliseconds
         """
+        assert clear_time > 0
         if self._status_clear_event_id != '':
             self._root.after_cancel(self._status_clear_event_id)
         self._status_bar_status['text'] = text
@@ -409,6 +451,8 @@ class PyDetexGUI(object):
         if self._cfg._last_opened_day_diff >= 7:
             self._root.after(1000, self._check_version_event)
         self._root.after(100, lambda: self._root.lift())
+        self._text_in.redraw()
+        self._text_out.redraw()
         self._root.mainloop()
 
     def _clear(self) -> None:
@@ -419,13 +463,17 @@ class PyDetexGUI(object):
         self._text_in.delete(0.0, tk.END)
         self._text_out.delete(0.0, tk.END)
         self._text_out['state'] = tk.DISABLED
-        self._copy_clip['state'] = tk.DISABLED
+        self._copy_clip_button['state'] = tk.DISABLED
+
         self._ready = False
         self._detect_language()
         self._status_bar_words['text'] = self._cfg.lang('status_words').format(0)
+
         self._status_clear()
         self._process_cursor_event()
         self._text_in.focus_force()
+        self._text_in.redraw()
+        self._text_out.redraw()
 
     @property
     def pipeline(self) -> pip.PipelineType:
@@ -440,25 +488,47 @@ class PyDetexGUI(object):
         """
         Process and call the pipeline.
         """
-        text = self._text_in.get(0.0, tk.END)
-        if text.strip() == '':
-            return self._clear()
         self._status(self._cfg.lang('status_processing'), True)
+        for btn in (self._process_button, self._process_clip_button,
+                    self._copy_clip_button, self._clear_button):
+            btn['state'] = tk.DISABLED
+        try:
+            self._root['cursor'] = 'wait'
+        except tk.TclError:
+            pass
+        self._root.after(50, lambda: self._process_inner())
 
+    def _process_inner(self) -> None:
+        """
+        Process called after.
+        """
+        text = self._text_in.get(0.0, tk.END)
         self._text_out['state'] = tk.NORMAL
-        self._copy_clip['state'] = tk.NORMAL
+        if self._clip:
+            self._copy_clip_button['state'] = tk.NORMAL
 
         # Font format
         font_format = self._cfg.get(self._cfg.CFG_OUTPUT_FONT_FORMAT)
+        PARSER_FONT_FORMAT['bold'] = FONT_TAGS['bold'] if font_format else ''
         PARSER_FONT_FORMAT['cite'] = FONT_TAGS['link'] if font_format else ''
         PARSER_FONT_FORMAT['equation'] = FONT_TAGS['italic'] if font_format else ''
+        PARSER_FONT_FORMAT['italic'] = FONT_TAGS['italic'] if font_format else ''
         PARSER_FONT_FORMAT['normal'] = FONT_TAGS['normal'] if font_format else ''
         PARSER_FONT_FORMAT['ref'] = FONT_TAGS['link'] if font_format else ''
         PARSER_FONT_FORMAT['tex_text_tag'] = FONT_TAGS['bold'] if font_format else ''
         PARSER_FONT_FORMAT['tex_text_tag_content'] = FONT_TAGS['italic'] if font_format else ''
 
         # Process the text and get the language
-        out = self.pipeline(text, self._detected_lang_tag)
+        # noinspection PyBroadException
+        try:
+            out = self.pipeline(text, self._detected_lang_tag)
+        except Exception:
+            err = self._cfg.lang('process_error').format(
+                pydetex.__url_bug_tracker__,
+                FONT_TAGS['error'] + traceback.format_exc()
+            )
+            self._text_out.insert_highlighted_text(FONT_TAGS['normal'] + err, True)
+            return self._process_final()
         words = len(self._tokenizer.tokenize(out))
         self._cfg.add_words(words)
 
@@ -485,21 +555,48 @@ class PyDetexGUI(object):
         # Insert the text
         self._text_out.insert_highlighted_text(out, True, font_format)
 
+        # Final
+        self._process_final(words)
+
+    def _process_final(self, words: int = 0) -> None:
+        """
+        Function executed after process finished.
+
+        :param words: Total processed words
+        """
         # Configure status
-        self._status_bar_words['text'] = self._cfg.lang('status_words').format(words)
+        try:
+            self._root['cursor'] = ''
+        except tk.TclError:
+            pass
+        self._process_button['state'] = tk.NORMAL
+        self._clear_button['state'] = tk.NORMAL
+        if self._clip:
+            self._process_clip_button['state'] = tk.NORMAL
+            self._copy_clip_button['state'] = tk.NORMAL
         self._ready = True
+        self._status_bar_words['text'] = self._cfg.lang('status_words').format(words)
         self._text_out['state'] = tk.DISABLED
+        self._text_out.redraw()
+
+        # Detect language to rewrite the status
         self._detect_language()
 
+        # If auto copy
+        if self._cfg.get(self._cfg.CFG_PROCESS_AUTO_COPY):
+            self._copy_to_clip()
+
     def _process_clip(self) -> None:
         """
         Process from clipboard. Tries with pooling.
         """
+        if not self._clip:
+            return
 
         def _paste():
             return pyperclip.paste()
 
-        self._status(self._cfg.lang('copy_from_clip'))
+        self._status(self._cfg.lang('copy_from_clip'), True)
         executor = concurrent.futures.ThreadPoolExecutor(1)
         future = executor.submit(_paste)
         try:
@@ -507,8 +604,11 @@ class PyDetexGUI(object):
         except concurrent.futures.TimeoutError:
             self._paste_timeout_error += 1
             if self._paste_timeout_error <= _MAX_PASTE_RETRY:
-                # print(f'Paste process failed (TimeoutError), retrying {self._paste_timeout_error}/{_MAX_PASTE_RETRY}')
-                self._root.after(100, self._process_clip)
+                print(f'Paste process failed (TimeoutError), retrying {self._paste_timeout_error}/{_MAX_PASTE_RETRY}')
+                try:
+                    self._root.after(100, self._process_clip_button)
+                except AttributeError:
+                    return self._process_clip()
             else:
                 self._paste_timeout_error = 0
                 error = f'Paste process failed after {_MAX_PASTE_RETRY} attempts'
@@ -516,10 +616,10 @@ class PyDetexGUI(object):
             return
 
         self._paste_timeout_error = 0
-        if text.strip() == '':
-            return
-        self._text_in.delete(0.0, tk.END)
-        self._text_in.insert(0.0, text)
+        text = text.strip()
+        if text == '':
+            return self._status(self._cfg.lang('clip_empty'), True)
+        self._insert_in(text, True)
         self._process()
 
     def _get_pipeline_results(self) -> str:
@@ -534,6 +634,9 @@ class PyDetexGUI(object):
         """
         Copy results to clip.
         """
+        if not self._clip:
+            return
+        self._status(self._cfg.lang('status_copy_to_clip'), clear=True)
         text = self._text_out.get(0.0, tk.END)
         pyperclip.copy(text)
 
@@ -545,7 +648,7 @@ class PyDetexGUI(object):
             self._settings_window.root.lift()
             return
         delta_h = 0 if ut.IS_OSX else 21
-        self._settings_window = gui_ut.SettingsWindow((375, 465 + delta_h), self._cfg)
+        self._settings_window = gui_ut.SettingsWindow((375, 519 + delta_h), self._cfg)
         self._settings_window.on_destroy = self._close_settings
         try:
             # self._settings_window.root.mainloop(1)
@@ -557,10 +660,11 @@ class PyDetexGUI(object):
         """
         Close settings.
         """
-        self._settings_window = None
         self._detect_language()
-        if self._ready:
-            self._process()
+        if self._ready and self._settings_window.saved:
+            # self._process()
+            pass
+        self._settings_window = None
 
     def _get_selected_word(self) -> str:
         """
